"""
This one is going to read csv.

For now, I am trying to read a talk outline and turn it into something
that will create a slideshow.

"""

import os
import sys
import csv
import pandas

import getopt


class Csv2Json

    def __init__(self):
        
        pass

    def interpret(self, infile):
        """ Process a file of rest and return json """

        # need row headings
        data = pandas.read_csv(infile)

        # FIXME find the right foo
        return json.dumps(data.foo())
            
                


if __name__ == '__main__':

    csv2json = Csv2Json()

    return csv2json.interpret(sys.stdin.read())




"""
Simple slide shower using eog.

TODO: would be nice if it did not give eog focus.
"""
import os
import time
import subprocess

from PIL import Image, ImageDraw

class SlideShow:

    def __init__(self):

        self.slides = []
        self.pos = 0
        self.wait = 0
        self.feh = None

    def interpret(self, msg):
        """ Create a slide show """

        self.captions = msg.get('captions', '.')
        
        for item in msg['slides']:
            self.add(item)

    def add(self, slide):

        self.slides.append(slide)

    def next(self):

        # send feh a signal to move to the next slide
        # 10 is SIGUSR1
        # this is byzantine, must be a better way to send a signal
        # from python
        os.system('kill -s 10 %d' % self.feh.pid)

    def show(self):
        
        #os.system('eog -g -w %s &' % image_file)
        #os.system('feh -F %s &' % image_file)
        slides = ' '.join([x.get('image', '') for x in self.slides])
        cmd = 'feh --scale-down --caption-path %s %s' % (
            self.captions, slides)
        self.feh = subprocess.Popen(cmd.split(' '))

    def set_duration(self, duration):
        """ Calculate how long each slide should show """
        fixed = sum(int(x.get('time', 0)) for x in self.slides)

        nfixed = len([x for x in self.slides if x.get('time', 0) > 0])

        unfixed = len(self.slides) - nfixed

        self.wait = max(1, int(duration / unfixed))

    def run(self):
        """ Run the show """

        self.show()

        if not self.wait:
            return
        
        for image in self.slides:
            wait = image.get('time', 0)
            wait = max(self.wait, wait)
            print('waiting %d seconds %s' % (
                wait, image.get('image', '')))
            yield image
            time.sleep(wait)
            self.next()

        



    



"""
Take a json string and turn it into python.

Result should be a list of dictionaries or a dictionary of lists or
whatever, per the json input.

World's simplest interpreter.
"""

import json

class Json2Py:

    def interpret(self, msg):

        return json.loads(msg)

    
x = Json2Py()

interpret = x.interpret



"""  Run whatever I'm doing at the time

Stuff in here probably belongs elsewhere.
"""

# run script from galleries folder

# fixme, make it md2py 
import md2slides
import slidelayout
import json2py
import py2json
import slide2png
import sys
import os

infile = os.path.expanduser(
    '~/devel/blog/stories/cat_modelling_with_python.md')

folder = '.'
if sys.argv[1:]:
    folder = sys.argv[1]
    
if sys.argv[2:]:
    infile = sys.argv[2]

mj = md2slides

msg = open(infile)

if infile.endswith('json'):
    mj = json2py
    msg = open(infile).read()

slides = mj.interpret(msg)

sl = slidelayout.SlideLayout()
slides = sl.interpret(dict(slides=slides))


print(py2json.interpret(slides))

s2png = slide2png.Slide2png()

msg = dict(slides=slides,
           gallery='../../blog/galleries')
s2png.interpret(msg)








"""
Create slides for a slideshow

Each slide is a heading plus a list of rows.

Each row is a list of text strings or image names.

This uses PIL to create an image for each slide.
"""
import os

from PIL import Image, ImageDraw, ImageFont

FONT = '/usr/share/fonts/TTF/Vera.ttf'
FONTSIZE = 36
WIDTH = 1024
HEIGHT = 768

class Slide2png:

    def __init__(self):

        self.pos = 0
        self.padding = 10
        self.cache = 'show'
        self.font = ImageFont.truetype(FONT, FONTSIZE)

    def interpret(self, msg):
        """ Load input """
        slides = msg.get('slides', [])
        self.cache = msg.get('folder', '.')
        self.gallery = msg.get('gallery', '..')

        # in case slides is a generator, turn it into a list
        # since I am going to go through it twice
        slides = [slide for slide in slides]

        # Write slides.txt with list of slides
        with open(self.cache + '/slides.txt', 'w') as logfile:
            for slide in slides:
                heading = slide['heading']['text']
                filename = self.get_image_name(heading)
                
                print('%s,%d' % (filename, slide.get('time', 0)),
                      file=logfile)

        # Now spin through slides again
        for slide in slides:

            image = self.draw_slide(slide)

            heading = slide['heading']['text']
            filename = self.get_image_name(heading)

            self.cache_image(filename, image)

        # fixme -- just return info in slides.txt as list of dicts
        return

    def draw_slide(self, slide):
        """ Return layout information for slide """

        image = Image.new('RGB', (WIDTH, HEIGHT), 'black')
        draw = ImageDraw.Draw(image)
        draw.font = self.font
        
        self.draw_slide_text(draw, slide)

        self.draw_slide_images(draw, slide, image)

        return image

    def draw_slide_text(self, draw, slide):

        heading = slide['heading']
        print(heading['text'])
        rows = slide['rows']

        
        left, top = heading['top'], heading['left']
        
        draw.text((left, top), heading['text'], fill='white')

        for row in rows:
            for item in row['items']:
                top, left = item['top'], item['left']
                
                print('tl', item['top'], item['left'])
                print('wh', item['width'], item['height'])

                print(item.get('text', ''))
                print(item.get('image', ''))

                text = item.get('text')

                if not text:
                    continue

                draw.text((left, top), text, fill='white')

                      
            print()

    def draw_slide_images(self, draw, slide, image):

        heading = slide['heading']
        rows = slide['rows']
        
        left, top = heading['top'], heading['left']
        
        for row in rows:
            for item in row['items']:
                top, left = item['top'], item['left']
                
                print('tl', item['top'], item['left'])
                print('wh', item['width'], item['height'])

                print(item.get('text', ''))
                print(item.get('image', ''))

                text = item.get('text')

                image_file = item.get('image')

                if not image_file: continue

                source = self.find_image(item)
                if source:
                    self.draw_image(image, item, source)
                else:
                    # no image, just use text
                    draw.text((left, top), text, fill='white')
                        
                      
            print()


    def find_image(self, item):
        """ Try and find the image file 

        some magic here would be good.

        FIXME move elsewhere and make so everyone can use.

        interpreter that finds things?
        """
        image_file = item['image']

        guess = os.path.join(self.gallery, image_file)
        if os.path.exists(guess):
            return guess

        return None
        

    def draw_image(self, image, item, source):
        """ Add an image to the image """
        top, left = item['top'], item['left']
        width, height = item['width'], item['height']
        image_file = item['image']
        
        img = Image.open(source)

        iwidth, iheight = img.size

        wratio = width / iwidth
        hratio = height / iheight

        print(width, height, iwidth, iheight)

        ratio = min(wratio, hratio)
        print(wratio, hratio, ratio)

        print('resizing %s %6.4f' % (image_file, ratio))
        
        img = img.resize((int(iwidth * ratio),
                          int(iheight * ratio)),
                         Image.ANTIALIAS)
        

        # get updated image size
        iwidth, iheight = img.size
        
        # Adjust top, left for actual size of image so centre
        # is in the same place as it would have been
        
        top += (height - iheight) // 2
        left += (width - iwidth) // 2
        
        # now paste the image
        print('pasting %s' % image_file)
        print(left, top)
        image.paste(img, (left, top))
        

    def slugify(self, name):
        """ Turn name into a slug suitable for an image file name """
        slug = ''
        last = ''
        for char in name.replace('#', '').lower().strip():
            if not char.isalnum():
                char = '_'

            if last == '_' and char == '_':
                continue

            slug += char
            last = char

        return slug
    
    def cache_image(self, name, image):
        
        
        with open(name, 'w') as slide:
            image.save(name)

        return name


    def get_image_name(self, label):

        return "%s/%s.png" % (self.cache, self.slugify(label))



"""  Run whatever I'm doing at the time

Stuff in here probably belongs elsewhere.
"""

# run script from galleries folder

# fixme, make it md2py 
import md2py
import json2py
import caption
import show
import sys
import os

infile = os.path.expanduser('~/devel/blog/stories/talk.rst')

folder = '.'
if sys.argv[1:]:
    folder = sys.argv[1]
    
if sys.argv[2:]:
    infile = sys.argv[2]

mj = md2py

msg = open(infile)

if infile.endswith('json'):
    mj = json2py
    msg = open(infile).read()

slides = mj.interpret(msg)

print(slides[:5])

ss = caption.SlideShow()
ss.interpret(dict(slides=slides, folder=folder))





"""
Add layout information to slides

Each slide is a heading plus a list of rows.

Each row is a list of text strings or image names.

This uses PIL to measure the size of text so it can
figure out where to lay everything out.

Input is a list of dictionaries.

Each dictionary describes a single slide.
"""
import os

from PIL import Image, ImageDraw, ImageFont

FONT = '/usr/share/fonts/TTF/Vera.ttf'
FONTSIZE = 36
WIDTH = 1024
HEIGHT = 768

class SlideLayout:

    def __init__(self):

        self.pos = 0
        self.padding = 10
        self.cache = 'show'
        self.font = ImageFont.truetype(FONT, FONTSIZE)

    def interpret(self, msg):
        """ Load input """
        slides = msg.get('slides', [])

        result = []
        for slide in slides:
            image = self.layout(slide)

            result.append(image)

        return result

    def layout(self, slide):
        """ Return layout information for slide """

        image = Image.new('RGB', (WIDTH, HEIGHT), 'black')
        draw = ImageDraw.Draw(image)
        draw.font = self.font
        
        self.vertical_layout(draw, slide)
        
        self.horizontal_layout(draw, slide)

        return slide


    def vertical_layout(self, draw, slide):
        """ Augment slide with vertical layout info """
        padding = self.padding
        heading = slide['heading']

        width, height = draw.textsize(heading['text'])
        top = padding
        left = padding

        # Calculate size and location of heading
        heading.update(dict(
            width = width,
            height = height,
            top = self.padding,
            left = self.padding))

        top += height + padding

        # count how many rows just text and how many have image
        rows = slide['rows']
        text_rows = 0
        image_rows = 0

        # calculate size of all text objects
        total_height = top
        for row in rows:

            row_height = 0

            images = 0
            for item in row['items']:
                if item.get('image'):
                    images += 1

                text = item.get('text')
                
                if text is None: continue
                    
                width, height = draw.textsize(text)

                item.update(dict(
                    width = width,
                    height = height))

                row_height = max(row_height, height)
                

            if images:
                image_rows += 1
                row['images'] = images
            else:
                row['height'] = row_height
                text_rows += 1

            total_height += row_height + padding
                
        # Calculate average height for image rows
        if image_rows:
            
            available = HEIGHT - total_height

            image_height = available // image_rows

            image_text_offset = image_height // 2

        # now spin through rows again setting top
        # (and height for images)
        for row in rows:
            text_top = top
            
            images = row.get('images', 0)
            if images:
                text_top += image_text_offset

            for item in row['items']:
                if item.get('text') is not None:
                    item['top'] = text_top
                else:
                    # image
                    item['top'] = top
                    item['height'] = image_height
                    row['height'] = image_height

            
            top += row.get('height', 0) + padding

        return

    def horizontal_layout(self, draw, slide):
        """ Augment slide with horizontal layout info """
        padding = self.padding
        heading = slide['heading']

        top = padding
        left = padding

        top += heading['height'] + padding

        rows = slide['rows']

        for row in rows:

            images = row.get('images', 0)

            items = row['items']
            
            used_width = sum(x.get('width', 0) for x in items)

            available_width = WIDTH - (
                used_width + ((1 + len(items)) * padding))

            if images:
                image_width = available_width // images

            # OK, now set left for all items and image_width for images
            left = padding
            for item in row['items']:

                if item.get('image'):
                    item['width'] = image_width

                item['left'] = left    

                left += item['width'] + padding

        return






"""
Create slides for a slideshow

TODO: would be nice if it did not give eog focus.
"""
import os

from PIL import Image, ImageDraw, ImageFont

FONT = '/usr/share/fonts/TTF/Vera.ttf'
FONTSIZE = 36
WIDTH = 1024
HEIGHT = 768

class SlideShow:

    def __init__(self):

        self.pos = 0
        self.cache = 'show'
        self.font = ImageFont.truetype(FONT, FONTSIZE)

    def interpret(self, msg):
        """ Load input """
        slides = msg.get('slides', [])
        self.cache = msg.get('folder')
        self.gallery = msg.get('gallery', '..')

        with open(self.cache + '/slides.txt', 'w') as logfile:
            for ix, item in enumerate(slides):
                image = self.prepare_image(item)
                filename = self.cache_image(item, image)

                text = item.get('caption', '')

                # do not write text for heading images
                if item.get('heading'):
                    text = ''
                    
                if text:
                    with open(filename + '.txt', 'w') as caption:
                        caption.write(text)
                print('%s,%d' % (filename, item.get('time', 0)), file=logfile)

    def prepare_image(self, slide):

        image = slide.get('image')
        caption = slide.get('caption')

        if caption is None:
            # use image name, without the suffic
            caption = os.path.splitext(image)[0]

            # convert _ to ' '
            caption = caption.replace('_', ' ')

            # save the caption
            slide['caption'] = caption
 
        # create image
        image_file = self.create_image(image, caption)

        return image_file

    def create_image(self, image_file, caption):
        """ Create an image with a caption """
        suffix = 'png'
        if image_file:
            img = Image.open(os.path.join(self.gallery, image_file))
            width, height = img.size
            ratio = width/WIDTH
            img = img.resize((int(width // ratio),
                              int(height // ratio)),
                             Image.ANTIALIAS)
        else:
            img = Image.new('RGB', (WIDTH, HEIGHT), 'black')
            image = self.add_caption(img, caption)
            
        image = img

        return image


    def cache_image(self, item, image):

        
        
        #name = "%s/slide%d.png" % (self.cache, ix)

        caption = item.get('image')
        if caption is None:
            caption = item.get('caption').split('\n')[0]
            
        caption = caption.split('/')[-1]
        caption = caption.replace(' ', '_')
        
        name = "%s/%s.png" % (self.cache, caption)
        
        with open(name, 'w') as slide:
            image.save(name)

        return name


    def add_caption(self, image, caption, colour=None):
        """ Add a caption to the image """

        if colour is None:
            colour = "white"
    
        width, height = image.size
        draw = ImageDraw.Draw(image)

        draw.font = self.font

        draw.font = self.font
        draw.text((width // 10, height//20), caption,
                  fill=colour)

        return image


        



    



"""
Take a python object and turn it into json.

World's simplest interpreter.
"""

import json

class Py2Json:

    def interpret(self, msg):

        return json.dumps(msg, indent=4)

x = Py2Json()

interpret = x.interpret



"""
This one is going to read markdown.

For now, I am trying to read a talk outline and turn it into something
that will create a slideshow.

This interpretter just turns markdown into a list of dictionaries.

The idea is then to feed this information into something like
caption.py which will create slides
"""

import os

class Mark2Slide:

    def __init__(self):

        pass

    def hash_count(self, line, pound=None):
        """ Count hashes (or other run of chars) at start of line

        >>> x = Mark2Slide()
        >>> x.hash_count('### hello')
        3

        >>> x = Mark2Slide()
        >>> x.hash_count(' hello', ' ')
        1

        """
        if pound is None:
            pound = '#'
            
        count = 0
        for c in line:
            if c != pound:
                break
            count += 1

        return count

    def generate_lines(self, infile):
        """ Split file into lines

        return dict with line=input, depth=n
        """
        pound = '#'
        
        for line in infile:

            heading = self.hash_count(line)

            indent = self.hash_count(line, pound=' ')

            yield dict(line=line.strip(),
                       heading=heading,
                       indent=indent)


    def generate_slides(self, infile):
        """ Process a file of rest and yield dictionaries """

        state = 0

        # each slide is a dict
        slide = {}

        last_heading = 0

        for item in self.generate_lines(infile):

            line = item['line']
            heading = item['heading']
            indent = item['indent']

            # Any heading is the heading for a new slide
            if heading:

                if slide and last_heading <= 1:
                    yield slide

                last_heading = heading
                    
                rows = []
                slide = dict(
                    heading = dict(text=line),
                    rows = rows)

                continue

            # Any block with more than one hash is a comment
            if last_heading > 1:
                continue

            if indent == 0 and line:
                # at a potential image
                rows.append(self.build_row(line))

            else:
                # Just add the line of text
                items = [dict(text=(' ' * indent) + line)]
                rows.append(dict(items=items))

        if slide:
            yield slide

    interpret = generate_slides
    

    def build_row(self, line):
        """ Line describes an image or images to show

        Returns a dict with a list of dicts of image names or text items

        Examples:

        # A single image to display
        >>> x.build_row('foo.png')
        [{'image': 'foo.png'}]

        # Two images with text in between:
        >>> x.build_row('foo.png or bar.jpg')
        [{'image': 'foo.png'}, {'text': 'or'}, {'image': 'bar.png'}]

        """

        items = []
        row = dict(items=items)
        fields = line.split(' ')

        image_exts = ['.png', '.jpg']

        # nothing there, carry on
        if not fields: return row

        for field in fields:

            ext = os.path.splitext(field)[-1]

            if ext.lower() in image_exts:
                items.append(
                    dict(image=field))

            else:
                items.append(
                    dict(text=field))

        return row


x = Mark2Slide()

interpret = x.interpret

if __name__ == '__main__':

    import sys
    m2s = Mark2Slide()

    slides = m2s.interpret(sys.stdin)

    for slide in slides:
        print(slide['heading'])

        rows = slide['rows']
        for row in rows:
            print(row)



"""  Run whatever I'm doing at the time

Stuff in here probably belongs elsewhere.
"""

# run script from galleries folder
import sys
import time

# fixme, make it md2py 
import md2py
import json2py
import show

import argparse

parser = argparse.ArgumentParser()

parser.add_argument('-t', '--time',
                    default=0, type=int,
                    help='time for slideshow in minutes')
parser.add_argument('-f', '--folder',
                    default='.',
                    help="folder of slides")
parser.add_argument('-s', '--slides',
                    default='slides.txt',
                    help='list of slides to show')
parser.add_argument('-w', '--wait',
                    default=0, type=int,
                    help='time to wait for slideshow')


args = parser.parse_args()

folder = args.folder
infile = args.slides
wait = args.wait


msg = open(infile)

mj = md2py

if infile.endswith('json'):
    mj = json2py
    msg = open(infile).read()

slides = mj.interpret(msg)

print("Number of slides:", len(slides))

print()
print(slides[0]['image'])
print()

ss = show.SlideShow()

ss.interpret(dict(slides=slides, captions=folder))

if args.time:
    ss.set_duration(args.time * 60)

time.sleep(wait)

for item in ss.run():
    # not sure what to do here
    pass



"""
This one is going to read markdown.

For now, I am trying to read a talk outline and turn it into something
that will create a slideshow.

"""

import os

class Mark2Py:

    def __init__(self):

        pass

    def interpret(self, infile):
        """ Process a file of rest and return list of dicts """

        data = []

        for record in self.generate_records(infile):
            data.append(record)

        return data

    def hash_count(self, line):
        """ Count hashes at start of line

        Sad but true..
        """
        count = 0
        for c in line:
            if c != '#':
                break
            count += 1

        return count

    def generate_lines(self, infile):
        """ Split file into lines

        return dict with line=input, depth=n
        """
        pound = '#'
        
        for line in infile:

            heading = 0

            if line.startswith(pound):
                heading = self.hash_count(line)

            yield dict(line=line, heading=heading)


    def generate_records(self, infile):
        """ Process a file of rest and yield dictionaries """

        state = 0
        record = {}

        for item in self.generate_lines(infile):

            line = item['line']
            heading = item['heading']
            
            # any Markdown heading is just a caption, no image
            if heading:
                print(line)
                record['heading'] = True
                record['caption'] = line[1:].strip()

                state = 'caption'
                continue

            if not line[0].isspace():
                # at a potential image
                if state == 'caption':
                    yield record
                    record = {}
                    state = 0

            if state == 'caption':
                record['caption'] += '\n' + line[:-1]
                continue

            fields = line.split(',')

            # nothing there, carry on
            if not fields: continue

            image = fields[0].strip()

            if not image: continue

            record['image'] = image

            try:
                time = float(fields[1])
            except:
                time = 0

            record['time'] = time

            try:
                caption = fields[2].strip()
            except:
                caption = None

            if caption:
                record['caption'] = caption

            # yield it if we have anything
            if record:
                yield record
                record = {}


                


x = Mark2Py()

interpret = x.interpret
            

